module App.Text.Explanations where
import Halogen.HTML as HH
import Bulma as Bulma

expl :: forall w i. Array (HH.HTML w i) -> HH.HTML w i
expl content = Bulma.div_content [ Bulma.explanation content ]

tokens :: forall w i. HH.HTML w i
tokens = HH.div_
  [ Bulma.h3 "What are tokens?"
  , expl [ Bulma.p """
          Tokens are a simple way to update a resource record (A or AAAA) with your current IP address.
          """
         ]
  , HH.p_ [ HH.text "Let's take an example: you have a A record (IPv4) pointing to your web server at home, "
          , HH.text "but your ISP changes your IP address from time to time. "
          , HH.text "You can ask for a token (which looks like "
          , HH.u_ [HH.text "53be0c45-61c4-4d29-8ae9-c2cc8767603d"]
          , HH.text ") for this specific entry, then make your server regularly visit the following website."
          ]
  , expl [ HH.p_ [ HH.text "https://beta.netlib.re/token-update/"
                 , HH.u_ [HH.text "<your-token>"]
                 ]
         ]
  , Bulma.p "For example: https://beta.netlib.re/token-update/53be0c45-61c4-4d29-8ae9-c2cc8767603d"
  , Bulma.hr
  , Bulma.h3 "How to automate the update of my IP address?"
  , Bulma.p "On Linux, you can make your computer access the update link with the following command."
  , expl [ Bulma.strong "wget https://beta.netlib.re/token-update/<token>" ]
  , Bulma.p """
    No need for a more complex program. This works just fine.
    And you can run this command every hour.
    For example, in your crontab (Linux and Unix related):
    """
  , expl [ Bulma.strong "0 * * * * wget <url>" ]
  , Bulma.p """
    Commands for other operating systems may differ, but you get the idea.
    """
  , Bulma.hr
  , Bulma.h3 "The obvious trap ⚠"
  , Bulma.p """
    Make sure to access the website using the related IP address.
    To update an IPv6 address (AAAA), force your application to access the URL using an IPv6 address.
    """
  , expl [ HH.p_ [ Bulma.strong "wget -6 <url>" ]
         , HH.p_ [ HH.text "To force the use of an IPv6 address." ]
         , HH.p_ [ Bulma.strong "wget -4 <url>" ]
         , HH.p_ [ HH.text "To force the use of an IPv4 address." ]
         ]
  ]

dkim_introduction :: forall w i. Array (HH.HTML w i)
dkim_introduction =
  [ Bulma.p """
    DKIM is a way to share a public signature key for the domain.
    This allows emails to be signed by the sender, and for the receiver to prove the origin of the mail.
    """
  , HH.p []
    [ HH.text """
        Default name is fine, change it only if you know what you are doing.
        For the configuration of your mail server, remember that your
      """
    , HH.u_ [HH.text "selector"]
    , HH.text " is "
    , Bulma.strong "default"
    , HH.text "."
    ]
  ]

dkim_default_algorithms :: forall w i. Array (HH.HTML w i)
dkim_default_algorithms =
  [ Bulma.p """
    Default values should be fine (RSA + SHA256), change them only if you know what you are doing.
    Just enter your public key.
  """
  ]

spf_introduction :: forall w i. Array (HH.HTML w i)
spf_introduction =
  [ HH.p []
    [ HH.text "Sender Policy Framework (SPF) is a way to tell "
    , HH.u_ [HH.text "other mail servers"]
    , HH.text " what are mail servers susceptible to send mails with email addresses from "
    , HH.u_ [HH.text "our domain"]
    , HH.text ". "
    ]
  , HH.p []
    [ HH.text """
    This way, we can mitigate spam.
    A server receiving a mail with our email address but coming from an IP address we didn't list as authorized will be discarded.
    This is not a bullet-proof technique, but it's simple enough and works great with the most basic forms of spam.
    """
    ]
  , HH.p []
    [ HH.text "A correctly configured domain with a mail server should only advertise the right IP addresses that can possibly send mails from the domain."
    ]
  , HH.p []
    [ HH.u_ [HH.text "Advice for novice users"]
    , HH.text """
      : default values should work great with simple domains.
      """
    ]
  ]

spf_default_behavior :: forall w i. Array (HH.HTML w i)
spf_default_behavior = [Bulma.p """
  What should someone do when receiving a mail with your email address but not from a listed domain or IP address?
  """
  , HH.text """
  By default, let's advise to drop the mail (a
  """
  , HH.u_ [HH.text "hard fail"]
  , HH.text """).
  The only way for DKIM to be really meaningful is to block any mail not coming from the intended email servers.
  Otherwise, it's just a statu quo, and the spamming will continue.
  """]

srv_introduction :: forall w i. Array (HH.HTML w i)
srv_introduction =
  [ Bulma.p "The SRV record is a DNS RR for specifying the location of services."
  , HH.p_ [ HH.text "Given a specific "
          , HH.u_ [HH.text "service name"]
          , HH.text " (which may be arbitrary) and a "
          , HH.u_ [HH.text "protocol"]
          , HH.text " (such as TCP or UDP), you can tell where the server is (address name and port). "
          , HH.text """
            Both the names of the service and the protocol are used to construct the name of the RR.
          """
          ]
  , HH.p_ [ HH.text "For example, for a service named "
          , HH.u_ [HH.text "voip"]
          , HH.text " and given that this service uses the TCP protocol, you can specify that the target is "
          , HH.u_ [HH.text "server1.example.com."]
          , HH.text "."
          ]
  ]
