module App.Validation.Login where

import Prelude

import Control.Alt ((<|>))
import Data.Either (Either(..))
import Data.Maybe (Maybe(..))
import Data.Validation.Semigroup (V, invalid, toEither)

import GenericParser.RFC5234 (alpha, digit)
import GenericParser.SomeParsers as SomeParsers
import GenericParser.Parser as G

data LoginParsingError
  = CannotParse
  | CannotEntirelyParse
  | Size Int Int Int

data Error
  = ParsingError (G.Error LoginParsingError)

min_login_size :: Int
min_login_size = 2
max_login_size :: Int
max_login_size = 50

parse :: forall e v. G.Parser e v -> String -> ((G.Error e) -> Error) -> V (Array Error) v
parse (G.Parser p) str c = case p { string: str, position: 0 } of
  Left x  -> invalid $ [c x]
  Right x -> pure x.result

login_parser :: G.Parser LoginParsingError String
login_parser = do
  input <- G.current_input
  _ <- G.many1 (alpha <|> digit) G.<:> \_ -> CannotParse
  _ <- SomeParsers.eof           G.<:> \_ -> CannotEntirelyParse
  pos <- G.current_position
  if between min_login_size max_login_size pos
  then pure input.string
  else G.errorParser (Just $ Size min_login_size max_login_size pos)

login :: String -> Either (Array Error) String
login s = toEither $ parse login_parser s ParsingError
