module App.Validation.Password where

import Prelude

import Control.Alt ((<|>))
import Data.Either (Either(..))
import Data.String.CodeUnits as CU
import Data.Maybe (Maybe(..))
import Data.Validation.Semigroup (V, invalid, toEither)

import GenericParser.RFC5234 (vchar)
import GenericParser.SomeParsers as SomeParsers
import GenericParser.Parser as G

data PasswordParsingError
  = CannotParse
  | CannotEntirelyParse
  | Size Int Int Int

data Error
  = ParsingError (G.Error PasswordParsingError)

min_password_size :: Int
min_password_size = 2
max_password_size :: Int
max_password_size = 100

parse :: forall e v. G.Parser e v -> String -> ((G.Error e) -> Error) -> V (Array Error) v
parse (G.Parser p) str c = case p { string: str, position: 0 } of
  Left x  -> invalid $ [c x]
  Right x -> pure x.result

password_parser :: G.Parser PasswordParsingError String
password_parser = do
  l <- G.many1 vchar <|> G.Parser \i -> G.failureError i.position (Just CannotParse)
  _ <- SomeParsers.eof <|> G.Parser \i -> G.failureError i.position (Just CannotEntirelyParse)
  pos <- G.current_position
  if pos < min_password_size || pos > max_password_size
  then G.Parser \i -> G.failureError i.position (Just $ Size min_password_size max_password_size pos)
  else pure $ CU.fromCharArray l

password :: String -> Either (Array Error) String
password s = toEither $ parse password_parser s ParsingError
